# Automatically generated - do not edit by hand


# noinspection SpellCheckingInspection
class Option(object):
    """Constants for option names."""

    CONFIG_REFRESH = "__config_refresh"
    NETPLAY_GAME = "__netplay_game"
    A2600_DATABASE = "a2600_database"
    A5200_DATABASE = "a5200_database"
    A7800_A78_HEADER = "a7800_a78_header"
    A7800_DATABASE = "a7800_database"
    A7800_MODEL = "a7800_model"
    A7800_PORT_1_TYPE = "a7800_port_1_type"
    A7800_PORT_2_TYPE = "a7800_port_2_type"
    ACCELERATOR = "accelerator"
    ACCELERATOR_MEMORY = "accelerator_memory"
    ACCELERATOR_ROM = "accelerator_rom"
    ACCURACY = "accuracy"
    ALIGN_X = "align_x"
    ALIGN_Y = "align_y"
    ALWAYS_ON_TOP = "always_on_top"
    AMIGA_DATABASE = "amiga_database"
    AMIGA_DRIVER = "amiga_driver"
    AMIGA_MODEL = "amiga_model"
    ARCADE_DATABASE = "arcade_database"
    ARCADE_FULLSCREEN = "arcade_fullscreen"
    ARCADE_INITIAL_FAVORITES = "arcade_initial_favorites"
    ARCADE_MAXIMIZED = "arcade_maximized"
    ARCADE_SEARCH = "arcade_search"
    ARCADE_SHUTDOWN = "arcade_shutdown"
    ARCADE_THEME = "arcade_theme"
    ASSUME_REFRESH_RATE = "assume_refresh_rate"
    ATARI_DATABASE = "atari_database"
    ATARI_MODEL = "atari_model"
    AUDIO_BUFFER_TARGET_BYTES = "audio_buffer_target_bytes"
    AUDIO_BUFFER_TARGET_SIZE = "audio_buffer_target_size"
    AUDIO_DRIVER = "audio_driver"
    AUDIO_FREQUENCY = "audio_frequency"
    AUTO_LOAD = "auto_load"
    AUTO_QUIT = "auto_quit"
    AUTOMATIC_INPUT_GRAB = "automatic_input_grab"
    BASE_DIR = "base_dir"
    BEZEL = "bezel"
    BLIZZARD_SCSI_KIT = "blizzard_scsi_kit"
    BORDER = "border"
    BSDSOCKET_LIBRARY = "bsdsocket_library"
    BUILTIN_CONFIGS = "builtin_configs"
    C64_DATABASE = "c64_database"
    C64_MODEL = "c64_model"
    C64_PALETTE = "c64_palette"
    C64_PORT_1_TYPE = "c64_port_1_type"
    C64_PORT_2_TYPE = "c64_port_2_type"
    CARTRIDGE_SLOT = "cartridge_slot"
    CD32_DATABASE = "cd32_database"
    CDFS = "cdfs"
    CDROM_DRIVE_0 = "cdrom_drive_0"
    CDROM_DRIVE_0_DELAY = "cdrom_drive_0_delay"
    CDROM_DRIVE_COUNT = "cdrom_drive_count"
    CDROM_IMAGE_0 = "cdrom_image_0"
    CDROM_IMAGE_1 = "cdrom_image_1"
    CDROM_IMAGE_10 = "cdrom_image_10"
    CDROM_IMAGE_11 = "cdrom_image_11"
    CDROM_IMAGE_12 = "cdrom_image_12"
    CDROM_IMAGE_13 = "cdrom_image_13"
    CDROM_IMAGE_14 = "cdrom_image_14"
    CDROM_IMAGE_15 = "cdrom_image_15"
    CDROM_IMAGE_16 = "cdrom_image_16"
    CDROM_IMAGE_17 = "cdrom_image_17"
    CDROM_IMAGE_18 = "cdrom_image_18"
    CDROM_IMAGE_19 = "cdrom_image_19"
    CDROM_IMAGE_2 = "cdrom_image_2"
    CDROM_IMAGE_3 = "cdrom_image_3"
    CDROM_IMAGE_4 = "cdrom_image_4"
    CDROM_IMAGE_5 = "cdrom_image_5"
    CDROM_IMAGE_6 = "cdrom_image_6"
    CDROM_IMAGE_7 = "cdrom_image_7"
    CDROM_IMAGE_8 = "cdrom_image_8"
    CDROM_IMAGE_9 = "cdrom_image_9"
    CDROMS_DIR = "cdroms_dir"
    CDTV_DATABASE = "cdtv_database"
    CHEATS = "cheats"
    CHIP_MEMORY = "chip_memory"
    CLIPBOARD_SHARING = "clipboard_sharing"
    COMMAND = "command"
    CONFIG_NAME = "config_name"
    CONSOLE_DEBUGGER = "console_debugger"
    CONTROLLERS_DIR = "controllers_dir"
    CPC_DATABASE = "cpc_database"
    CPU = "cpu"
    CPU_IDLE = "cpu_idle"
    CPUBOARD_FLASH_EXT_FILE = "cpuboard_flash_ext_file"
    CPUBOARD_FLASH_FILE = "cpuboard_flash_file"
    CROP = "crop"
    CUE_SHEETS = "cue_sheets"
    CURSOR = "cursor"
    DATABASE_AUTH = "database_auth"
    DATABASE_EMAIL = "database_email"
    DATABASE_FEATURE = "database_feature"
    DATABASE_LOCKER = "database_locker"
    DATABASE_PASSWORD = "database_password"
    DATABASE_SERVER = "database_server"
    DATABASE_SHOW_ADULT = "database_show_adult"
    DATABASE_SHOW_GAMES = "database_show_games"
    DATABASE_SHOW_UNPUBLISHED = "database_show_unpublished"
    DATABASE_USERNAME = "database_username"
    DETERMINISTIC = "deterministic"
    DEVELOPER_MODE = "developer_mode"
    DEVICE_ID = "device_id"
    DISABLE_ASPECT_CORRECTION = "disable_aspect_correction"
    DONGLE_TYPE = "dongle_type"
    DOS_DATABASE = "dos_database"
    DOS_EMULATOR = "dos_emulator"
    DOSBOX_CPU_CORE = "dosbox_cpu_core"
    DOSBOX_CPU_CPUTYPE = "dosbox_cpu_cputype"
    DOSBOX_CPU_CYCLES = "dosbox_cpu_cycles"
    DOSBOX_GUS_GUS = "dosbox_gus_gus"
    DOSBOX_MACHINE = "dosbox_machine"
    DOSBOX_MEMSIZE = "dosbox_memsize"
    DOSBOX_SBLASTER_IRQ = "dosbox_sblaster_irq"
    DOSBOX_SBLASTER_OPLRATE = "dosbox_sblaster_oplrate"
    DOSBOX_SBLASTER_SBBASE = "dosbox_sblaster_sbbase"
    DOSBOX_SBLASTER_SBTYPE = "dosbox_sblaster_sbtype"
    EFFECT = "effect"
    EMULATOR_PID_FILE = "emulator_pid_file"
    END_CONFIG = "end_config"
    EXPECT_VERSION = "expect_version"
    FADE_IN_DURATION = "fade_in_duration"
    FADE_OUT_DURATION = "fade_out_duration"
    FAKE_JOYSTICKS = "fake_joysticks"
    FAKE_VERSION = "fake_version"
    FAST_MEMORY = "fast_memory"
    FILE_LIST = "file_list"
    FLASH_MEMORY_DIR = "flash_memory_dir"
    FLOPPIES_DIR = "floppies_dir"
    FLOPPY_DRIVE_0 = "floppy_drive_0"
    FLOPPY_DRIVE_0_SOUNDS = "floppy_drive_0_sounds"
    FLOPPY_DRIVE_1 = "floppy_drive_1"
    FLOPPY_DRIVE_1_SOUNDS = "floppy_drive_1_sounds"
    FLOPPY_DRIVE_2 = "floppy_drive_2"
    FLOPPY_DRIVE_2_SOUNDS = "floppy_drive_2_sounds"
    FLOPPY_DRIVE_3 = "floppy_drive_3"
    FLOPPY_DRIVE_3_SOUNDS = "floppy_drive_3_sounds"
    FLOPPY_DRIVE_COUNT = "floppy_drive_count"
    FLOPPY_DRIVE_SPEED = "floppy_drive_speed"
    FLOPPY_DRIVE_VOLUME = "floppy_drive_volume"
    FLOPPY_DRIVE_VOLUME_EMPTY = "floppy_drive_volume_empty"
    FLOPPY_IMAGE_0 = "floppy_image_0"
    FLOPPY_IMAGE_1 = "floppy_image_1"
    FLOPPY_IMAGE_10 = "floppy_image_10"
    FLOPPY_IMAGE_11 = "floppy_image_11"
    FLOPPY_IMAGE_12 = "floppy_image_12"
    FLOPPY_IMAGE_13 = "floppy_image_13"
    FLOPPY_IMAGE_14 = "floppy_image_14"
    FLOPPY_IMAGE_15 = "floppy_image_15"
    FLOPPY_IMAGE_16 = "floppy_image_16"
    FLOPPY_IMAGE_17 = "floppy_image_17"
    FLOPPY_IMAGE_18 = "floppy_image_18"
    FLOPPY_IMAGE_19 = "floppy_image_19"
    FLOPPY_IMAGE_2 = "floppy_image_2"
    FLOPPY_IMAGE_3 = "floppy_image_3"
    FLOPPY_IMAGE_4 = "floppy_image_4"
    FLOPPY_IMAGE_5 = "floppy_image_5"
    FLOPPY_IMAGE_6 = "floppy_image_6"
    FLOPPY_IMAGE_7 = "floppy_image_7"
    FLOPPY_IMAGE_8 = "floppy_image_8"
    FLOPPY_IMAGE_9 = "floppy_image_9"
    FLOPPY_OVERLAYS_DIR = "floppy_overlays_dir"
    FMV_ROM = "fmv_rom"
    FORCE_ASPECT = "force_aspect"
    FPU = "fpu"
    FRAME_TIME = "frame_time"
    FREEZER_CARTRIDGE = "freezer_cartridge"
    FSAA = "fsaa"
    FULL_KEYBOARD = "full_keyboard"
    FULLSCREEN = "fullscreen"
    FULLSCREEN_HEIGHT = "fullscreen_height"
    FULLSCREEN_MODE = "fullscreen_mode"
    FULLSCREEN_WIDTH = "fullscreen_width"
    G_SYNC = "g_sync"
    GAME_LIST_UUID = "game_list_uuid"
    GAME_MODE = "game_mode"
    GAME_NAME = "game_name"
    GB_DATABASE = "gb_database"
    GB_PORT_1_TYPE = "gb_port_1_type"
    GBA_DATABASE = "gba_database"
    GBA_GAMMA = "gba_gamma"
    GBA_PORT_1_TYPE = "gba_port_1_type"
    GBC_DATABASE = "gbc_database"
    GBC_PORT_1_TYPE = "gbc_port_1_type"
    GOVERNOR_WARNING = "governor_warning"
    GRAB_INPUT = "grab_input"
    GRAPHICS_CARD = "graphics_card"
    GRAPHICS_CARD_ROM = "graphics_card_rom"
    GRAPHICS_MEMORY = "graphics_memory"
    HARD_DRIVE_0 = "hard_drive_0"
    HARD_DRIVE_0_CONTROLLER = "hard_drive_0_controller"
    HARD_DRIVE_0_FILE_SYSTEM = "hard_drive_0_file_system"
    HARD_DRIVE_0_LABEL = "hard_drive_0_label"
    HARD_DRIVE_0_PRIORITY = "hard_drive_0_priority"
    HARD_DRIVE_0_READ_ONLY = "hard_drive_0_read_only"
    HARD_DRIVE_0_TYPE = "hard_drive_0_type"
    HARD_DRIVE_1 = "hard_drive_1"
    HARD_DRIVE_2 = "hard_drive_2"
    HARD_DRIVE_3 = "hard_drive_3"
    HARD_DRIVE_4 = "hard_drive_4"
    HARD_DRIVE_5 = "hard_drive_5"
    HARD_DRIVE_6 = "hard_drive_6"
    HARD_DRIVE_7 = "hard_drive_7"
    HARD_DRIVE_8 = "hard_drive_8"
    HARD_DRIVE_9 = "hard_drive_9"
    HARD_DRIVES_DIR = "hard_drives_dir"
    INITIAL_INPUT_GRAB = "initial_input_grab"
    INPUT_GRAB = "input_grab"
    INT_ACCELERATOR_NAME = "int_accelerator_name"
    INT_BOGOMEM_SIZE = "int_bogomem_size"
    INT_CHIPMEM_SIZE = "int_chipmem_size"
    INT_CHIPSET_NAME = "int_chipset_name"
    INT_CPU_NAME = "int_cpu_name"
    INT_CPUBOARDMEM1_SIZE = "int_cpuboardmem1_size"
    INT_DEFAULT_FLOPPY_TYPE = "int_default_floppy_type"
    INT_FASTMEM_SIZE = "int_fastmem_size"
    INT_GRAPHICS_CARD_BUS = "int_graphics_card_bus"
    INT_GRAPHICS_CARD_NAME = "int_graphics_card_name"
    INT_KICKSTART_EXT_SHA1 = "int_kickstart_ext_sha1"
    INT_KICKSTART_REVISION = "int_kickstart_revision"
    INT_KICKSTART_SHA1 = "int_kickstart_sha1"
    INT_KICKSTART_VERSION = "int_kickstart_version"
    INT_MBRESMEM_LOW_SIZE = "int_mbresmem_low_size"
    INT_MODEL = "int_model"
    INT_MODEL_NAME = "int_model_name"
    INT_PPC_MODEL = "int_ppc_model"
    INT_UAE_BOOT_ROM = "int_uae_boot_rom"
    INT_Z3CHIPMEM_SIZE = "int_z3chipmem_size"
    INT_Z3FASTMEM_SIZE = "int_z3fastmem_size"
    IRC_NICK = "irc_nick"
    IRC_SERVER = "irc_server"
    JIT_COMPILER = "jit_compiler"
    JIT_MEMORY = "jit_memory"
    JOYSTICK_PORT_0 = "joystick_port_0"
    JOYSTICK_PORT_0_AUTOFIRE = "joystick_port_0_autofire"
    JOYSTICK_PORT_0_AUTOSWITCH = "joystick_port_0_autoswitch"
    JOYSTICK_PORT_0_MODE = "joystick_port_0_mode"
    JOYSTICK_PORT_1 = "joystick_port_1"
    JOYSTICK_PORT_1_AUTOFIRE = "joystick_port_1_autofire"
    JOYSTICK_PORT_1_MODE = "joystick_port_1_mode"
    JOYSTICK_PORT_2 = "joystick_port_2"
    JOYSTICK_PORT_2_AUTOFIRE = "joystick_port_2_autofire"
    JOYSTICK_PORT_2_MODE = "joystick_port_2_mode"
    JOYSTICK_PORT_3 = "joystick_port_3"
    JOYSTICK_PORT_3_AUTOFIRE = "joystick_port_3_autofire"
    JOYSTICK_PORT_3_MODE = "joystick_port_3_mode"
    KEEP_ASPECT = "keep_aspect"
    KEYBOARD_INPUT_GRAB = "keyboard_input_grab"
    KEYBOARD_KEY_BACKSLASH = "keyboard_key_backslash"
    KEYBOARD_KEY_EQUALS = "keyboard_key_equals"
    KEYBOARD_KEY_INSERT = "keyboard_key_insert"
    KEYBOARD_KEY_LESS = "keyboard_key_less"
    KICKSTART_EXT_FILE = "kickstart_ext_file"
    KICKSTART_FILE = "kickstart_file"
    KICKSTART_SETUP = "kickstart_setup"
    KICKSTARTS_DIR = "kickstarts_dir"
    LANGUAGE = "language"
    LAUNCHER_CLOSE_BUTTONS = "launcher_close_buttons"
    LAUNCHER_CONFIG_FEATURE = "launcher_config_feature"
    LAUNCHER_FONT_SIZE = "launcher_font_size"
    LAUNCHER_LOG_CONFIG = "launcher_log_config"
    LAUNCHER_SETUP_WIZARD_FEATURE = "launcher_setup_wizard_feature"
    LAUNCHER_THEME = "launcher_theme"
    LINE_DOUBLING = "line_doubling"
    LOAD_STATE = "load_state"
    LOCALIZATION = "localization"
    LOG_AUTOSCALE = "log_autoscale"
    LOG_BSDSOCKET = "log_bsdsocket"
    LOG_FLUSH = "log_flush"
    LOG_INPUT = "log_input"
    LOG_QUERY_PLANS = "log_query_plans"
    LOGS_DIR = "logs_dir"
    LOW_LATENCY_VSYNC = "low_latency_vsync"
    LOW_RESOLUTION = "low_resolution"
    MAME_ARTWORK = "mame_artwork"
    MEDNAFEN_AUDIO_BUFFER = "mednafen_audio_buffer"
    MEDNAFEN_AUDIO_DRIVER = "mednafen_audio_driver"
    MEDNAFEN_DEINTERLACER = "mednafen_deinterlacer"
    MEDNAFEN_TEMPORAL_BLUR = "mednafen_temporal_blur"
    MIDDLE_CLICK_UNGRAB = "middle_click_ungrab"
    MIN_FIRST_LINE_NTSC = "min_first_line_ntsc"
    MIN_FIRST_LINE_PAL = "min_first_line_pal"
    MODEL = "model"
    MONITOR = "monitor"
    MOTHERBOARD_RAM = "motherboard_ram"
    MOUSE_INTEGRATION = "mouse_integration"
    MOUSE_SPEED = "mouse_speed"
    MSX_DATABASE = "msx_database"
    N64_DATABASE = "n64_database"
    NDS_DATABASE = "nds_database"
    NEOGEO_DATABASE = "neogeo_database"
    NEOGEO_MODEL = "neogeo_model"
    NEOGEO_PORT_1_TYPE = "neogeo_port_1_type"
    NEOGEO_PORT_2_TYPE = "neogeo_port_2_type"
    NES_DATABASE = "nes_database"
    NES_DRIVER = "nes_driver"
    NES_EMULATOR = "nes_emulator"
    NES_INES_HEADER = "nes_ines_header"
    NES_MODEL = "nes_model"
    NES_PORT_1_TYPE = "nes_port_1_type"
    NES_PORT_2_TYPE = "nes_port_2_type"
    NES_PORT_3_TYPE = "nes_port_3_type"
    NES_PORT_4_TYPE = "nes_port_4_type"
    NETPLAY_FEATURE = "netplay_feature"
    NETPLAY_PASSWORD = "netplay_password"
    NETPLAY_PORT = "netplay_port"
    NETPLAY_SERVER = "netplay_server"
    NETPLAY_TAG = "netplay_tag"
    NETWORK_CARD = "network_card"
    NGC_DATABASE = "ngc_database"
    NOTIFICATION_DURATION = "notification_duration"
    NTSC_MODE = "ntsc_mode"
    OFFLINE_MODE = "offline_mode"
    ORIENTATION = "orientation"
    PLATFORM = "platform"
    PLATFORMS_FEATURE = "platforms_feature"
    PRIMARY_JOYSTICK = "primary_joystick"
    PSX_DATABASE = "psx_database"
    PSX_MODEL = "psx_model"
    PSX_PORT_1_TYPE = "psx_port_1_type"
    PSX_PORT_2_TYPE = "psx_port_2_type"
    PSX_PRELOAD = "psx_preload"
    QUICK_SETTINGS = "quick_settings"
    QUICK_SETTINGS_OPTIONS = "quick_settings_options"
    RAW_INPUT = "raw_input"
    REFRESH_RATE = "refresh_rate"
    RELATIVE_PATHS = "relative_paths"
    RELATIVE_TEMP_FEATURE = "relative_temp_feature"
    RETROARCH_AUDIO_BUFFER = "retroarch_audio_buffer"
    RTG_SCANLINES = "rtg_scanlines"
    RTG_VIEWPORT = "rtg_viewport"
    SAVE_DISK = "save_disk"
    SAVE_STATE_COMPRESSION = "save_state_compression"
    SAVE_STATES = "save_states"
    SAVE_STATES_DIR = "save_states_dir"
    SBI_DATA = "sbi_data"
    SBLASTER_IRQ = "sblaster_irq"
    SCALE = "scale"
    SCALE_X = "scale_x"
    SCALE_Y = "scale_y"
    SCANLINES = "scanlines"
    SCANLINES_DARK = "scanlines_dark"
    SCANLINES_LIGHT = "scanlines_light"
    SCREENSHOTS_OUTPUT_DIR = "screenshots_output_dir"
    SCREENSHOTS_OUTPUT_MASK = "screenshots_output_mask"
    SCREENSHOTS_OUTPUT_PREFIX = "screenshots_output_prefix"
    SEARCH_PATH = "search_path"
    SECONDARY_JOYSTICK = "secondary_joystick"
    SERIAL_PORT = "serial_port"
    SGG_DATABASE = "sgg_database"
    SGG_PORT_1_TYPE = "sgg_port_1_type"
    SHADER = "shader"
    SLOW_MEMORY = "slow_memory"
    SMD_DATABASE = "smd_database"
    SMD_EMULATOR = "smd_emulator"
    SMD_MODEL = "smd_model"
    SMD_PORT_1_TYPE = "smd_port_1_type"
    SMD_PORT_2_TYPE = "smd_port_2_type"
    SMOOTHING = "smoothing"
    SMS_DATABASE = "sms_database"
    SMS_DRIVER = "sms_driver"
    SNAPSHOT_FILE = "snapshot_file"
    SNES_DATABASE = "snes_database"
    SNES_PORT_1_TYPE = "snes_port_1_type"
    SNES_PORT_2_TYPE = "snes_port_2_type"
    SOUND_CARD = "sound_card"
    SRAM_TYPE = "sram_type"
    STATE_DIR = "state_dir"
    STATE_DIR_NAME = "state_dir_name"
    STEREO_SEPARATION = "stereo_separation"
    STRETCH = "stretch"
    SUB_TITLE = "sub_title"
    SWAP_CTRL_KEYS = "swap_ctrl_keys"
    SYNC = "sync"
    TAPE_DRIVE_0 = "tape_drive_0"
    TEXTURE_FILTER = "texture_filter"
    TEXTURE_FORMAT = "texture_format"
    TG16_DATABASE = "tg16_database"
    TGCD_DATABASE = "tgcd_database"
    THEME = "theme"
    THEME_CUSTOM_0 = "theme_custom_0"
    THEME_CUSTOM_0_X = "theme_custom_0_x"
    THEME_CUSTOM_0_Y = "theme_custom_0_y"
    THEME_FADE_COLOR = "theme_fade_color"
    THEME_FLOOR_COLOR_1 = "theme_floor_color_1"
    THEME_FLOOR_COLOR_2 = "theme_floor_color_2"
    THEME_HEADING_COLOR = "theme_heading_color"
    THEME_OVERLAY_IMAGE = "theme_overlay_image"
    THEME_WALL_COLOR_1 = "theme_wall_color_1"
    THEME_WALL_COLOR_2 = "theme_wall_color_2"
    TITLE = "title"
    TURBO_LOAD = "turbo_load"
    UAE_A1000RAM = "uae_a1000ram"
    UAE_A2065 = "uae_a2065"
    UAE_A3000MEM_SIZE = "uae_a3000mem_size"
    UAE_AGNUS_BLTBUSYBUG = "uae_agnus_bltbusybug"
    UAE_BOGOMEM_FAST = "uae_bogomem_fast"
    UAE_BOGOMEM_SIZE = "uae_bogomem_size"
    UAE_BSDSOCKET_EMU = "uae_bsdsocket_emu"
    UAE_CART = "uae_cart"
    UAE_CART_FILE = "uae_cart_file"
    UAE_CART_INTERNAL = "uae_cart_internal"
    UAE_CD32C2P = "uae_cd32c2p"
    UAE_CD32CD = "uae_cd32cd"
    UAE_CD32FMV = "uae_cd32fmv"
    UAE_CD32NVRAM = "uae_cd32nvram"
    UAE_CDTVCD = "uae_cdtvcd"
    UAE_CDTVRAM = "uae_cdtvram"
    UAE_CDTVRAMCARD = "uae_cdtvramcard"
    UAE_CHIPMEM_SIZE = "uae_chipmem_size"
    UAE_CHIPSET = "uae_chipset"
    UAE_CHIPSET_COMPATIBLE = "uae_chipset_compatible"
    UAE_CIAATOD = "uae_ciaatod"
    UAE_COLLISION_LEVEL = "uae_collision_level"
    UAE_CPU_24BIT_ADDRESSING = "uae_cpu_24bit_addressing"
    UAE_CPU_COMPATIBLE = "uae_cpu_compatible"
    UAE_CPU_FREQUENCY = "uae_cpu_frequency"
    UAE_CPU_MODEL = "uae_cpu_model"
    UAE_CPU_MULTIPLIER = "uae_cpu_multiplier"
    UAE_CPU_RESET_PAUSE = "uae_cpu_reset_pause"
    UAE_CPU_SPEED = "uae_cpu_speed"
    UAE_CPU_THROTTLE = "uae_cpu_throttle"
    UAE_CPU_TYPE = "uae_cpu_type"
    UAE_CPUBOARD_TYPE = "uae_cpuboard_type"
    UAE_CPUBOARDMEM1_SIZE = "uae_cpuboardmem1_size"
    UAE_DENISE_NOEHB = "uae_denise_noehb"
    UAE_DONGLE = "uae_dongle"
    UAE_FASTMEM2_SIZE = "uae_fastmem2_size"
    UAE_FASTMEM2_SIZE_K = "uae_fastmem2_size_k"
    UAE_FASTMEM_AUTOCONFIG = "uae_fastmem_autoconfig"
    UAE_FASTMEM_SIZE = "uae_fastmem_size"
    UAE_FASTMEM_SIZE_K = "uae_fastmem_size_k"
    UAE_FATGARY = "uae_fatgary"
    UAE_FILESYS_INJECT_ICONS = "uae_filesys_inject_icons"
    UAE_FILESYS_INJECT_ICONS_DRAWER = "uae_filesys_inject_icons_drawer"
    UAE_FILESYS_INJECT_ICONS_PROJECT = "uae_filesys_inject_icons_project"
    UAE_FILESYS_INJECT_ICONS_TOOL = "uae_filesys_inject_icons_tool"
    UAE_FINEGRAIN_CPU_SPEED = "uae_finegrain_cpu_speed"
    UAE_FLOPPY0 = "uae_floppy0"
    UAE_FLOPPY0TYPE = "uae_floppy0type"
    UAE_FLOPPY1 = "uae_floppy1"
    UAE_FLOPPY1TYPE = "uae_floppy1type"
    UAE_FLOPPY2 = "uae_floppy2"
    UAE_FLOPPY2TYPE = "uae_floppy2type"
    UAE_FLOPPY3 = "uae_floppy3"
    UAE_FLOPPY3TYPE = "uae_floppy3type"
    UAE_FORCE_0X10000000_Z3 = "uae_force_0x10000000_z3"
    UAE_FPU_MODEL = "uae_fpu_model"
    UAE_GFX_LINEMODE = "uae_gfx_linemode"
    UAE_GFXCARD_HARDWARE_SPRITE = "uae_gfxcard_hardware_sprite"
    UAE_GFXCARD_HARDWARE_VBLANK = "uae_gfxcard_hardware_vblank"
    UAE_GFXCARD_SIZE = "uae_gfxcard_size"
    UAE_GFXCARD_TYPE = "uae_gfxcard_type"
    UAE_ICS_AGNUS = "uae_ics_agnus"
    UAE_IDE = "uae_ide"
    UAE_IMMEDIATE_BLITS = "uae_immediate_blits"
    UAE_JOYPORT0 = "uae_joyport0"
    UAE_JOYPORT0AUTOFIRE = "uae_joyport0autofire"
    UAE_JOYPORT0MODE = "uae_joyport0mode"
    UAE_JOYPORT1 = "uae_joyport1"
    UAE_JOYPORT1AUTOFIRE = "uae_joyport1autofire"
    UAE_JOYPORT1MODE = "uae_joyport1mode"
    UAE_KSMIRROR_A8 = "uae_ksmirror_a8"
    UAE_KSMIRROR_E0 = "uae_ksmirror_e0"
    UAE_MAPROM = "uae_maprom"
    UAE_MBRESMEM_SIZE = "uae_mbresmem_size"
    UAE_MEGACHIPMEM_SIZE = "uae_megachipmem_size"
    UAE_MMU_MODEL = "uae_mmu_model"
    UAE_NATIVE_CODE = "uae_native_code"
    UAE_PCMCIA = "uae_pcmcia"
    UAE_PPC_CPU_IDLE = "uae_ppc_cpu_idle"
    UAE_PPC_MODEL = "uae_ppc_model"
    UAE_RAMSEY = "uae_ramsey"
    UAE_RESETWARNING = "uae_resetwarning"
    UAE_RTC = "uae_rtc"
    UAE_SANA2 = "uae_sana2"
    UAE_SCSI_A2091 = "uae_scsi_a2091"
    UAE_SCSI_A3000 = "uae_scsi_a3000"
    UAE_SCSI_A4000T = "uae_scsi_a4000t"
    UAE_SCSI_A4091 = "uae_scsi_a4091"
    UAE_SCSI_CDTV = "uae_scsi_cdtv"
    UAE_SCSIDEV_MODE = "uae_scsidev_mode"
    UAE_SLIRP_IMPLEMENTATION = "uae_slirp_implementation"
    UAE_SLIRP_PORTS = "uae_slirp_ports"
    UAE_SLIRP_REDIR = "uae_slirp_redir"
    UAE_SOUND_INTERPOL = "uae_sound_interpol"
    UAE_SOUND_OUTPUT = "uae_sound_output"
    UAE_SOUND_STEREO_SEPARATION = "uae_sound_stereo_separation"
    UAE_TOCCATA = "uae_toccata"
    UAE_WAITING_BLITS = "uae_waiting_blits"
    UAE_Z3CHIPMEM_SIZE = "uae_z3chipmem_size"
    UAE_Z3MAPPING = "uae_z3mapping"
    UAE_Z3MEM2_SIZE = "uae_z3mem2_size"
    UAE_Z3MEM_SIZE = "uae_z3mem_size"
    UAE_Z3MEM_START = "uae_z3mem_start"
    UAEGFX_CARD = "uaegfx_card"
    UAELOG = "uaelog"
    UAEM_WRITE_FLAGS = "uaem_write_flags"
    UAENATIVE_LIBRARY = "uaenative_library"
    UNSAFE_SAVE_STATES = "unsafe_save_states"
    VARIANT_NAME = "variant_name"
    VICE_AUDIO_DRIVER = "vice_audio_driver"
    VIDEO_FORMAT = "video_format"
    VIDEO_STANDARD = "video_standard"
    VIDEO_SYNC = "video_sync"
    VIDEO_SYNC_METHOD = "video_sync_method"
    VIEWPORT = "viewport"
    VOLUME = "volume"
    WARP_MODE = "warp_mode"
    WHDLOAD_BOOT_DIR = "whdload_boot_dir"
    WHDLOAD_MODEL = "whdload_model"
    WHDLOAD_PRELOAD = "whdload_preload"
    WHDLOAD_QUIT_KEY = "whdload_quit_key"
    WHDLOAD_SPLASH_DELAY = "whdload_splash_delay"
    WINDOW_BORDER = "window_border"
    WINDOW_HEIGHT = "window_height"
    WINDOW_HIDDEN = "window_hidden"
    WINDOW_MINIMIZED = "window_minimized"
    WINDOW_RESIZABLE = "window_resizable"
    WINDOW_WIDTH = "window_width"
    WINDOW_X = "window_x"
    WINDOW_Y = "window_y"
    WORKBENCH = "workbench"
    WRITABLE_FLOPPY_IMAGES = "writable_floppy_images"
    X_WHDLOAD_ARGS = "x_whdload_args"
    X_WHDLOAD_VERSION = "x_whdload_version"
    ZOOM = "zoom"
    ZORRO_III_MEMORY = "zorro_iii_memory"
    ZXS_DATABASE = "zxs_database"
    ZXS_DRIVER = "zxs_driver"
    ZXS_MODEL = "zxs_model"
